@@ -1,7 +1,7 @@
 """
 banyan_base.py
 
- Copyright (c) 2016-2019 Alan Yorinks All right reserved.
+ Copyright (c) 2016-2021 Alan Yorinks All right reserved.
 
  Python Banyan is free software; you can redistribute it and/or
  modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
@@ -53,6 +53,7 @@ class BanyanBase(object):
 
     """
 
+    # noinspection PyBroadException
     def __init__(self, back_plane_ip_address=None, subscriber_port='43125',
                  publisher_port='43124', process_name='None', loop_time=.1, numpy=False,
                  external_message_processor=None, receive_loop_idle_addition=None,
@@ -81,10 +82,12 @@ class BanyanBase(object):
         :param receive_loop_idle_addition: an external method called in the idle section
                                            of the receive loop
 
-        :param connect_time: a short delay to allow the component to connect to the Backplane
+        :param connect_time: a short delay to allow the component to connect
+                             to the Backplane
         """
 
-        # call to super allows this class to be used in multiple inheritance scenarios when needed
+        # call to super allows this class to be used in multiple
+        # inheritance scenarios when needed
         super(BanyanBase, self).__init__()
 
         self.backplane_exists = False
@@ -124,8 +127,13 @@ class BanyanBase(object):
             # determine this computer's IP address
             s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
             # use the google dns
-            s.connect(('8.8.8.8', 0))
-            self.back_plane_ip_address = s.getsockname()[0]
+            try:
+                s.connect(('8.8.8.8', 1))
+                self.back_plane_ip_address = s.getsockname()[0]
+            except Exception:
+                self.back_plane_ip_address = '127.0.0.1'
+            finally:
+                s.close()
 
         self.subscriber_port = subscriber_port
         self.publisher_port = publisher_port
@@ -140,18 +148,34 @@ class BanyanBase(object):
         print('************************************************************')
 
         # establish the zeromq sub and pub sockets and connect to the backplane
-        self.context = zmq.Context()
-        self.subscriber = self.context.socket(zmq.SUB)
+        self.my_context = zmq.Context()
+        self.subscriber = self.my_context.socket(zmq.SUB)
         connect_string = "tcp://" + self.back_plane_ip_address + ':' + self.subscriber_port
         self.subscriber.connect(connect_string)
 
-        self.publisher = self.context.socket(zmq.PUB)
+        self.publisher = self.my_context.socket(zmq.PUB)
         connect_string = "tcp://" + self.back_plane_ip_address + ':' + self.publisher_port
         self.publisher.connect(connect_string)
 
         # Allow enough time for the TCP connection to the Backplane complete.
         time.sleep(self.connect_time)
 
+    def get_subscriber(self):
+        """
+        Retrieve the zmq subscriber object
+
+        :return: self.subscriber
+        """
+        return self.subscriber
+
+    def get_publisher(self):
+        """
+        Retrieve the zmq subscriber object
+
+        :return: self.subscriber
+        """
+        return self.publisher
+
     def set_subscriber_topic(self, topic):
         """
         This method sets a subscriber topic.
@@ -246,7 +270,7 @@ class BanyanBase(object):
         """
         self.publisher.close()
         self.subscriber.close()
-        self.context.term()
+        self.my_context.term()
 
 # When creating a derived component, replicate the code below and replace
 # banyan_base with a name of your choice.
@@ -289,16 +313,16 @@ class BanyanBase(object):
 #     except KeyboardInterrupt:
 #         sys.exit()
 #
-#     # signal handler function called when Control-C occurs
-#     # noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
-#     def signal_handler(signal, frame):
-#         print("Control-C detected. See you soon.")
-#         app.clean_up()
-#         sys.exit(0)
+# signal handler function called when Control-C occurs
+# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
+# def signal_handler(sig, frame):
+#     print('Exiting Through Signal Handler')
+#     raise KeyboardInterrupt
 #
-#     # listen for SIGINT
-#     signal.signal(signal.SIGINT, signal_handler)
-#     signal.signal(signal.SIGTERM, signal_handler)
+#
+# # listen for SIGINT
+# signal.signal(signal.SIGINT, signal_handler)
+# signal.signal(signal.SIGTERM, signal_handler)
 #
 #
 # if __name__ == '__main__':
