@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 """
- Copyright (c) 2017 Alan Yorinks - All Rights Reserved.
+ Copyright (c) 2018-2019 Alan Yorinks - All Rights Reserved.
 
 
  Python Banyan is free software; you can redistribute it and/or
@@ -31,9 +31,11 @@ from subprocess import Popen
 
 import psutil
 from apscheduler.schedulers.background import BackgroundScheduler
+
 from python_banyan.banyan_base import BanyanBase
 
 
+# noinspection PyTypeChecker
 class BLS(BanyanBase):
     """
     This is the banyan launcher server. It reads a user .csv
@@ -49,15 +51,15 @@ class BLS(BanyanBase):
         topic = local if the process should be run on this computer.
                 For clients, it is the topic they will be listening for
                 to get their launch instructions
-        append_bp_address = yes or no. If yes a -b commmand line option with
-                            the IP adress of this computer is appended to the
+        append_bp_address = yes or no. If yes a -b command line option with
+                            the IP address of this computer is appended to the
                             command string.
         auto_restart = yes or no. If yes, if the process dies it will be restarted.
 
     """
 
     def __init__(self, subscriber_port='43125', publisher_port='43124',
-                 descriptor_file='launch.csv'):
+                 descriptor_file='launch.csv', ):
         """
         Initialize the launch server
         :param subscriber_port: backplane subscriber port
@@ -130,6 +132,7 @@ class BLS(BanyanBase):
 
         # prepare an entry for the launch_db table
         new_entry['command_string'] = 'backplane'
+        new_entry['wait'] = '0'
 
         # add the backplane entry to the launch database
         self.launch_db.append(new_entry)
@@ -153,6 +156,7 @@ class BLS(BanyanBase):
                 new_entry['append_bp_address'] = row['append_bp_address']
                 new_entry['command_string'] = row['command_string']
                 new_entry['spawn'] = row['spawn']
+                new_entry['wait'] = row['wait']
 
                 # if the topic is not 'local' then create a reply topic
                 # and check if the command needs to have a -b option appended
@@ -168,9 +172,6 @@ class BLS(BanyanBase):
                 # add the populated record to launch_db
                 self.launch_db.append(new_entry)
 
-        # allow time for zmq connections to establish
-        time.sleep(.3)
-
         # display pid of backplane either just invoked or already running
         if not bp_string:
             print('backplane                           PID = {}'.format(str(self.proc.pid)))
@@ -186,6 +187,12 @@ class BLS(BanyanBase):
                     self.spawn_local(idx)
                 else:
                     self.publish_payload(record, record['topic'])
+            try:
+                if int(record['wait']) > 0:
+                    time.sleep(int(record['wait']))
+            except TypeError:
+                print(record)
+                raise
 
         # start the background scheduler to periodically run check_processes and confirm methods
         self.scheduler = BackgroundScheduler()
@@ -204,7 +211,6 @@ class BLS(BanyanBase):
         except (KeyboardInterrupt, SystemExit):
             self.clean_up()
 
-
     def spawn_local(self, idx):
         """
         This method launches processes that are needed to run on this computer
@@ -303,33 +309,36 @@ class BLS(BanyanBase):
                         # check to see if the process in the launch_db is in this list
                         # status = None
 
-                        if record['process'].pid in pids:
-                            proc = psutil.Process(record['process'].pid)
-                            status = proc.status()
-                            # print('command: ' + record['command_string'] + 'status: ' + status)
+                        try:
+                            if record['process'].pid in pids:
+                                proc = psutil.Process(record['process'].pid)
+                                status = proc.status()
+                                # print('command: ' + record['command_string'] + 'status: ' + status)
 
-                            if status == psutil.STATUS_SLEEPING:
-                                continue
+                                if status == psutil.STATUS_SLEEPING:
+                                    continue
 
-                            if not status:
-                                status = psutil.STATUS_ZOMBIE
+                                if not status:
+                                    status = psutil.STATUS_ZOMBIE
 
-                            if status == psutil.STATUS_ZOMBIE:
+                                if status == psutil.STATUS_ZOMBIE:
 
-                                log_string = '{:35} PID = {} DIED'.format(record['command_string'],
-                                                                          str(record['process'].pid))
-                                print(log_string)
+                                    log_string = '{:35} PID = {} DIED'.format(record['command_string'],
+                                                                              str(record['process'].pid))
+                                    print(log_string)
 
-                                logger = logging.getLogger()
-                                logger.error(log_string)
+                                    logger = logging.getLogger()
+                                    logger.error(log_string)
 
-                                # reset its process, and process ID
-                                record['process'] = None
-                                record['process_id'] = None
+                                    # reset its process, and process ID
+                                    record['process'] = None
+                                    record['process_id'] = None
 
-                                # do we need to restart it?
-                                if record['auto_restart'] == 'yes':
-                                    self.spawn_local(x)
+                                    # do we need to restart it?
+                                    if record['auto_restart'] == 'yes':
+                                        self.spawn_local(x)
+                        except AttributeError:
+                            pass
 
     def incoming_message_processing(self, topic, payload):
         """
@@ -339,16 +348,19 @@ class BLS(BanyanBase):
         :param payload: Message Data
         :return:
         """
+        print(topic)
         if topic == 'killall':
             self.clean_up()
 
         # update the launch_db record for the remote process so that we do not send additional launch commands
         for idx, record in enumerate(self.launch_db):
-            # if no launch id has alreary been assigned, update the record and print a message
-            if not record['launch_id']:
+            # if no launch id has already been assigned, update the record and print a message
+            # if not record['launch_id']:
+            # is this an acknowledgement from the remote launch?
+            if record['launch_id']:
                 if record['launch_id'] == payload['launch_id']:
                     self.launch_db[idx]['process_id'] = payload['process_id']
-                    print('{}: {:35} PID = {}'.format(topic, record['command_string'], str(self.proc.pid)))
+                    print('{}: {:25} PID = {}'.format(topic, record['command_string'], str(payload['process_id'])))
                     break
 
     def confirm_remote(self):
@@ -359,8 +371,9 @@ class BLS(BanyanBase):
         """
 
         for idx, record in enumerate(self.launch_db):
-            if not record['process']:
-                self.publish_payload(record, record['topic'])
+            if not record['process_id']:
+                if record['topic'] != 'local':
+                    self.publish_payload(record, record['topic'])
 
     def clean_up(self):
         """
@@ -372,12 +385,18 @@ class BLS(BanyanBase):
 
         for idx, record in enumerate(self.launch_db):
             if record['process']:
+
                 print('{:35} PID = {} KILLED'.format(record['command_string'], str(record['process'].pid)))
                 try:
                     proc = psutil.Process(record['process'].pid)
+                    for c in proc.children(recursive=True):
+                        c.kill()
                     proc.kill()
                     record['process'] = None
                     record['process_id'] = None
+                    # this is necessary for killing the command windows in Windows
+                    if proc.name() == record['command_string']:
+                        proc.kill()
                 except psutil.NoSuchProcess:
                     pass
 
@@ -392,40 +411,33 @@ def bls():
     parser = argparse.ArgumentParser()
 
     parser.add_argument("-f", dest="descriptor_file", default='launch.csv',
-                        help="Publisher IP port")
+                        help="csv descriptor file")
     parser.add_argument("-p", dest="publisher_port", default='43124',
                         help="Publisher IP port")
-    parser.add_argument("-r", dest="relaunch_all", default='False',
-                        help="Relaunch All If Backplane Dies")
     parser.add_argument("-s", dest="subscriber_port", default='43125',
                         help="Subscriber IP port")
 
     args = parser.parse_args()
 
-    if args.relaunch_all == 'True':
-        args.relaunch_all = True
-    else:
-        args.relaunch_all = False
     kw_options = {
         'publisher_port': args.publisher_port,
         'subscriber_port': args.subscriber_port,
-        'descriptor_file': args.descriptor_file}
+        'descriptor_file': args.descriptor_file, }
 
     # replace with the name of your class
     BLS(**kw_options)
 
-    # signal handler function called when Control-C occurs
-    # noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
-    def signal_handler(signal, frame):
-        print("Control-C detected. See you soon.")
-        sys.exit(0)
 
-    # listen for SIGINT
-    signal.signal(signal.SIGINT, signal_handler)
-    signal.signal(signal.SIGTERM, signal_handler)
+# signal handler function called when Control-C occurs
+# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
+def signal_handler(sig, frame):
+    print('Exiting Through Signal Handler')
+    raise KeyboardInterrupt
 
-    # signal.signal(signal.SIGKILL, signal_handler)
 
+# listen for SIGINT
+signal.signal(signal.SIGINT, signal_handler)
+signal.signal(signal.SIGTERM, signal_handler)
 
 if __name__ == '__main__':
     # replace with name of function you defined above
